///////////////////////////////////////////////////////////////////////////////////////
//
// TestClient.cpp : Sample code to demonstrate writing to and reading from a TCP-MODBUS 
//					compliant device using WA_TCP_MODBUS
//.
//

#include "stdafx.h"
#include "stdio.h"
#include "conio.h"
#include "TestClient.h"

///////////////////////////////////////////////////////////////////////////////////////////
//
// Here are two sample addresses taken from a CLS multi-loop controller's MODBUS data table
//
#define CLS_DATA_ADDRESS_PROPORTIONAL_BAND	40001
#define CLS_DATA_ADDRESS_DIGITAL_OUTPUT		907

////////////////////////////////////////////////////////////////////////
// This will always be the TCP port number used by the TCP-MODBUS device
//
#define MODBUS_HOST_PORT					502


//////////////////////////////////////////////////////////
//
// Display error encountered while transmitting TCP string
//
void displayTransmissionError(short errCode)
{
	switch(errCode)
	{
		case HOST_REJECTED_DATA_LENGTH: printf("\nToo many items transmitted, maximum number=%d",MAX_MODBUS_DATA_LENGTH);
			break;
		case HOST_REJECTED_DATA_ADDRESS: printf("\nData Address outside is outside valid MODBUS range: 1-50,000");
			break;
		case DEVICE_REJECTED_DATA_ADDRESS: printf("\nData Address specified is not valid for this device");
			break;
		case DEVICE_REJECTED_DATA_VALUE: printf("\nData value is out of the valid range for this address");
			break;
		case ERROR_SENDING_STRING: printf("\nWinSock API error generated upon transmitting TCP string");
			break;
		case ERROR_ZERO_BYTES_RECEIVED: printf("\nTCP timeout, zero bytes received");
			break;
	}
}

////////////////////////////////////////////////////////////
//
// Display error initializing and opening TCP Windows Socket
//
void displaySocketInitializationError(short errCode)
{
	switch(errCode)
	{
		case ERROR_SOCK_INITIALIZATION: printf("\nError initializing Windows Socket DLL");
			break;
		case ERROR_SOCK_CREATION: printf("\nError creating Windows Socket");
			break;
		case ERROR_RESOLVING_HOST_NAME: printf("\nError: Can not resolve specified host name");
			break;
		case ERROR_ESTABLISHING_CONNECTION: printf("\nError, can not establish TCP connection");
			break;
	}
}

int main(int argc, char* argv[])
{
	/////////////////////////////////////////////////////////
	// Open and initialize Windows Socket connection
	// In this case we are talking to a CLS device via a Watlow TCP-MODBUS Gateway with 
	// Server Address = em000108
	// The port a TCP-MODBUS server uses is always 502
	//
	// The controller of interest has a communication (node) address = 17
	//
	printf("\nInitializing Windows Socket...");
	short result = WA_openTCPModbusConnection("em000108", MODBUS_HOST_PORT);

	short data[200];

	//////////////////////////////////////////////////////
	// Set the values in an array to be sent to controller
	//
	data[0] = 35;
	data[1] = 36;
	data[2] = 37;
	data[3] = 38;
	data[4] = 39;


	///////////////////////////////////////////////////////////////////////////////
	// Write the same values to the CLS' Proportional Band of the CLS over and over
	//
	while (!_kbhit())
	{
		result = WA_writeTCPModbusDevice(17, CLS_DATA_ADDRESS_PROPORTIONAL_BAND, data, 5);
		if (result == GOOD)
			printf("\nValues written: %d %d %d %d %d", data[0], data[1], data[2], data[3], data[4]);
		else
			displayTransmissionError(result);
	}
	getch();

	/////////////////////////////////////////////////////////
	// Repeatedly read the values in CLS' Proportional Band
	//
	while (!_kbhit())
	{
		result = WA_readTCPModbusDevice(17, CLS_DATA_ADDRESS_PROPORTIONAL_BAND, data, 2);
		if (result == GOOD)
			printf("\nValues read: %d %d %d %d %d", data[0], data[1], data[2], data[3], data[4]);
		else
			displayTransmissionError(result);
	}
	getch();

	/////////////////////////////////////////////////////////
	// Repeatedly read the values in the CLS' digital outputs
	//
	while (!_kbhit())
	{
		result = WA_readTCPModbusDevice(17, CLS_DATA_ADDRESS_DIGITAL_OUTPUT, data, 20);
		if (result == GOOD)
			printf("\nValues read: %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d", 
			data[0], data[1], data[2], data[3], data[4],data[5], data[6], data[7], data[8], data[9],
			data[10], data[11], data[12], data[13], data[14],data[15], data[16], data[17], data[18], data[19]);
		else
			displayTransmissionError(result);
	}


	///////////////////////////////////////////////////////////////////////////////////////////////////
	// Write a bogus value (100) to Series 96 controller residing at communication node adress #9
	// A write to data adress 45001 should elicit a response from the device indicating an error condition
	//
	data[0] = 100;
	while (!_kbhit())
	{
		result = WA_writeTCPModbusDevice(9, 45001, data, 1);
		if (result == GOOD)
			printf("\nValues written: %d %d %d %d %d", data[0], data[1], data[2], data[3], data[4]);
		else
			displayTransmissionError(result);
	}
	getch();

	///////////////////////////////////////////////////////////////////////////////////////////////////
	// Close connection now that all communication is done. No need to close connection during pollintg
	//.
	WA_closeTCPModbusConnection();

	return 0;
}


